pub enum RetryError {
CallFailed(ErrorCause),
StatusUnknown(ErrorCause),
}Expand description
An error type for retried calls.
This enum distinguishes between cases where we know the call failed and cases where we cannot determine the final status of the call.
Variants
CallFailed(ErrorCause)
We know that the call failed, and there is no point in retrying
StatusUnknown(ErrorCause)
A fatal error. We don’t know whether the call failed, but there is no point in retrying.